
import os, sys

from java.lang import Exception
from java.io import ByteArrayOutputStream, FileInputStream, \
                    InputStreamReader, FileOutputStream, PrintWriter, \
                    StringWriter, File
from javax.swing import JFileChooser, JOptionPane, JDialog, \
                        JComboBox, Box, BorderFactory, JButton
from java.awt import Dimension
from javax.swing.filechooser import FileFilter
from java.net import URL

from com.hp.hpl.jena.rdf.model import ModelFactory
from com.hp.hpl.jena.rdql import Query, QueryEngine, QueryResultsFormatter
from com.hp.hpl.jena.ontology import OntModelSpec

from rdf_filechooser import RDFModelChooser


#------------  MODEL -----------------------------------
class StringVar:

    def __init__(self, textComponent):
        self.txtComp = textComponent

    def get(self):
        return self.txtComp.text

    def set(self, newString):
        self.txtComp.text = newString



class QueryHistory:

    def __init__(self):
        self.list = []
        self.current = None

    def addQuery(self, queryStr):
        self.list.append(queryStr)
        self.current = len(self.list) - 1
    
    def getPreviousQuery(self):
        if self.list != [] and self.current > 0:
            self.current = self.current - 1
            return self.list[self.current]
        else:
            return None

    def getNextQuery(self):
        max = len(self.list) - 1
        if self.list != [] and self.current < max:
            self.current = self.current + 1
            return self.list[self.current]
        else:
            return None


        
#------------  CONTROLLER -----------------------------------

class Controller:

    def __init__(self):
        self.query = None
        self.output = None
        self.modelName = None
        self.byteStream = ByteArrayOutputStream()
        self.model = None
        self.frame = None
        self.history = QueryHistory()
        self.openPath = os.getcwd()
        self.urlHistory = ['http://www.w3.org/1999/02/22-rdf-syntax-ns#',
                           'http://www.w3.org/2000/01/rdf-schema#']

    def openFile(self, evt):
##        self.closeModel(None)
            
        chooser = RDFModelChooser(self.openPath)
        chooser.dialogTitle = 'Open RDF Model'
        result = chooser.showOpenDialog(self.frame)
        
        if result == JFileChooser.APPROVE_OPTION:
            self.openPath = chooser.selectedFile.parent
            result = chooser.selectedFile.path
            ext = result[ result.rfind('.')+1 : ]

            result = File(result).toURI().toString()

            self.modelName.set(result)

            self.openModel(result, ext)

    def openURL(self, evt):
##        self.closeModel(None)

        dlg = OpenURLDialog(self.frame, self.urlHistory)

        url = dlg.getURL()

        if url != None:
            if not url in self.urlHistory:
                self.urlHistory = [url] + self.urlHistory
            self.modelName.set(url)
            self.openModel(dlg.getURL())
        
            

    def openModel(self, urlStr, ext):
        url = URL(urlStr)
        lang = ''
        if ext == 'rdf':  lang = 'RDF/XML'
        elif ext == 'nt': lang = 'N-TRIPLE'
        elif ext == 'n3': lang = 'N3'        

##        self.model = ModelFactory.createDefaultModel()
        
        if self.model == None:
            self.model = ModelFactory.createOntologyModel(
                OntModelSpec.RDFS_MEM_RDFS_INF , None )

        fin = None
        try:
            fin = InputStreamReader(url.openStream())
            self.model.read(fin, None, lang)
        except Exception, e:
            errStr = 'Error opening %s: %s' % (urlStr, e.message)
            self.output.set(errStr)

        if fin != None:  fin.close()

    def closeModel(self, evt):
        if (self.model != None):
            self.model.close()
            self.model = None
            self.modelName.set('')
            self.output.set('')

    def clearQuery(self, evt):
        self.query.set('')

    def evalQuery(self, evt):
        self.output.set('')

        try:
            query = Query(self.query.get())
            
            if self.model != None:
                query.setSource(self.model)
        
            qe = QueryEngine(query)
            results = qe.exec()
            fmt = QueryResultsFormatter(results)

            sw = StringWriter()
            pw = PrintWriter(sw)
            fmt.printAll(pw, " | ")
            pw.flush()
            fmt.close()	
            results.close()
        except Exception, e:
            self.output.set('Error: ' + e.message)
        else:
            self.output.set(sw.toString())
            self.history.addQuery(self.query.get())

        

    def allTriples(self, evt):
        queryStr = 'SELECT ?s, ?p, ?o' + '\n' + \
                   'WHERE (?s ?p ?o)'
        
        self.query.set(queryStr)


    def showAbout(self, evt):
        aboutStr = '%s\n\n%s\n%s' % (
            'RDQL-Front',
            'Author:  Feihong Hsu',
            'Email:  fhsu@cs.uic.edu'
        )
        
        JOptionPane.showMessageDialog(self.frame, aboutStr)


    def prevQuery(self, evt):
        result = self.history.getPreviousQuery()
        if result != None:
            self.query.set(result)
        
    def nextQuery(self, evt):
        result = self.history.getNextQuery()
        if result != None:
            self.query.set(result)

    def export(self, evt):
        if self.model == None:
            self.output.set('Error:  No model for export')
        else:
            chooser = RDFModelChooser(self.openPath)
            chooser.dialogTitle = 'Export RDF Model'
            result = chooser.showSaveDialog(self.frame)
            
            if result == JFileChooser.APPROVE_OPTION:
                result = chooser.selectedFile.path
                
                lang = chooser.fileFilter.description
                lang = lang[0 : lang.find('files')-1].strip()

                fout = FileOutputStream(result)
                self.model.write(fout, lang)

                fout.close()                

    def quit(self, evt):
        sys.exit(0)


class OpenURLDialog(JDialog):
        def __init__(self, owner=None, urlList=[]):
            JDialog.__init__(self, owner,
                             title='Open URL',
                             modal=1)

            self.cbox = JComboBox(urlList,
                                  editable=1,
                                  alignmentX=0,
                                  preferredSize=Dimension(450,25))
            self.url = None
            
            self.createWidgets()
            self.pack()
            self.show()
            

        def createWidgets(self):
            box = Box.createVerticalBox()
            box.border = BorderFactory.createEmptyBorder(10,10,10,10)

            # self.cbox declared in __init__
            box.add(self.cbox)

            box.add(self.createButtonPane())            

            self.contentPane.add(box)

        def createButtonPane(self):
            box = Box.createHorizontalBox()
            box.alignmentX = 0

            box.add(Box.createHorizontalGlue())

            btn = JButton('OK', actionPerformed=self.ok)
            box.add(btn)

            btn = JButton('Cancel', actionPerformed=self.cancel)
            box.add(btn)
            
            return box

        def ok(self, evt):
            self.url = self.cbox.selectedItem
            self.hide()

        def cancel(self, evt):
            self.hide()
            
        def getURL(self):
            return self.url
