
from javax.swing import JFrame, JLabel, JButton, JTextArea, JTextField, \
                        Box, BorderFactory, JScrollPane, KeyStroke, \
                        JMenu, JMenuBar, JMenuItem, JSeparator, \
                        JSplitPane, JRadioButton, ButtonGroup
from java.awt import Dimension, Font
from java.awt.event import KeyEvent, ActionEvent

from rdql_control import Controller, StringVar



class App(JFrame):

    def __init__(self):
        JFrame.__init__(self, title='RDQL-Front',
                              defaultCloseOperation=JFrame.EXIT_ON_CLOSE)

        self.control = Controller()

        # Bind self.control.frame to self
        self.control.frame = self
        
        self.createWidgets()
        self.pack()
        self.show()

    def createWidgets(self):
        self.setJMenuBar(self.createMenuBar())
        
        b = Box.createVerticalBox()
        b.alignmentX = 0
        b.border = BorderFactory.createEmptyBorder(20,10,10,10)

        b.add(self.createModelPane())

        b.add(Box.createRigidArea(Dimension(0,15)))

        b.add(self.createSplitPane())
        
        self.contentPane.add(b);

    def createMenuBar(self):
        keyStr = "KeyStroke.getKeyStroke(KeyEvent.VK_%s, " + \
                 "ActionEvent.CTRL_MASK)"
        keyFn = lambda k, str=keyStr: eval(str % k)
        
        ctrlO = KeyStroke.getKeyStroke(KeyEvent.VK_O,
                                       ActionEvent.CTRL_MASK)
        ctrlW = KeyStroke.getKeyStroke(KeyEvent.VK_W,
                                       ActionEvent.CTRL_MASK)
        ctrlK = KeyStroke.getKeyStroke(KeyEvent.VK_K,
                                       ActionEvent.CTRL_MASK)
        ctrlUp = KeyStroke.getKeyStroke(KeyEvent.VK_UP,
                                        ActionEvent.CTRL_MASK)
        ctrlDn = KeyStroke.getKeyStroke(KeyEvent.VK_DOWN,
                                        ActionEvent.CTRL_MASK)
        ctrlT = KeyStroke.getKeyStroke(KeyEvent.VK_T,
                                        ActionEvent.CTRL_MASK)
                               
        ctrlL = keyFn('L')
   
        fileMenu = JMenu('File')
        menuItems = [JMenuItem('Open File...',
                        actionPerformed=self.control.openFile,
                        accelerator=keyFn('O')),
                     JMenuItem('Open URL...',
                        actionPerformed=self.control.openURL,
                        accelerator=keyFn('U')),
                     JMenuItem('Close',
                        actionPerformed=self.control.closeModel,
                        accelerator=keyFn('W')),
                     JMenuItem('Export...',
                        actionPerformed=self.control.export,
                        accelerator=keyFn('E')),
                     JMenuItem('Quit',
                        actionPerformed=self.control.quit)
        ]
        for item in menuItems:  fileMenu.add(item)       
                        

        queryMenu = JMenu('Query')
        menuItems = [JMenuItem('Clear',
                            actionPerformed=self.control.clearQuery,
                            accelerator=keyFn('L')),
                     JMenuItem('Evaluate',
                            actionPerformed=self.control.evalQuery,
                            accelerator=keyFn('K')),
                     JSeparator(),
                     JMenuItem('Add namespaces...'),
                     JMenuItem('All triples',
                            actionPerformed=self.control.allTriples,
                            accelerator=keyFn('T')),
                     JSeparator(),
                     JMenuItem('Previous query',
                            actionPerformed=self.control.prevQuery,
                            accelerator=keyFn('UP')),
                     JMenuItem('Next query',
                            actionPerformed=self.control.nextQuery,
                            accelerator=keyFn('DOWN'))
        ]
        for item in menuItems:  queryMenu.add(item)

        helpMenu = JMenu('Help')
        menuItems = [JMenuItem('Contents...'),
                     JMenuItem('About...',
                          actionPerformed=self.control.showAbout)
        ]
        for item in menuItems:  helpMenu.add(item) 
        
        menubar = JMenuBar()
        menubar.add(fileMenu)
        menubar.add(queryMenu)
        menubar.add(helpMenu)

        return menubar

    def createModelPane(self):
        b = Box.createHorizontalBox()
        b.alignmentX = 0
        b.add(JLabel('Model'))

        b.add(Box.createRigidArea(Dimension(5,0)))

        txt = JTextField(30,
                         alignmentX=0,
                         editable=0,
                         maximumSize=Dimension(500, 25))
        b.add(txt)
        
        # Bind to self.control.modelName
        self.control.modelName = StringVar(txt)
            
        return b

    def createSplitPane(self):
        upperPane = self.createQueryPane()
        lowerPane = self.createOutputPane()
        splitPane = JSplitPane(JSplitPane.VERTICAL_SPLIT,
                               upperPane, lowerPane);
        splitPane.oneTouchExpandable = 1;
        splitPane.dividerLocation = 150;

        return splitPane
    
    def createQueryPane(self):
        b = Box.createVerticalBox()
        b.add(JLabel('Query'))
        
        txt = JTextArea(6, 80,
                        tabSize=4,
                        font=Font('Monospaced', Font.PLAIN, 12)  )
        scr = JScrollPane(txt,
                          alignmentX=0)
        b.add(scr)

        # Bind to self.control.query
        self.control.query = StringVar(txt)
        
        return b


    def createButtonPane(self):
        b = Box.createHorizontalBox()
        b.alignmentX = 0
        b.add(Box.createHorizontalGlue())

        btn = JButton('Clear', actionPerformed=self.control.clearQuery)
        b.add(btn)

        b.add(Box.createRigidArea(Dimension(5,0)))

        btn = JButton('Evaluate', actionPerformed=self.control.evalQuery)
        b.add(btn)

        return b

    def createOutputPane(self):
        b = Box.createVerticalBox()
        b.add(JLabel('Output'))
        
        txt = JTextArea(15, 80,
                        tabSize=4,
                        editable=0,
                        font=Font('Monospaced', Font.PLAIN, 12) )
        scr = JScrollPane(txt, alignmentX=0)
        b.add(scr)

        # Bind to self.control.output
        self.control.output = StringVar(txt)
        
        return b




if __name__ == '__main__':
    a = App();
