
from com.hp.hpl.jena.rdf.model import ModelFactory
from com.hp.hpl.jena.rdql import Query, QueryEngine, QueryResultsFormatter

from java.io import PrintWriter, StringWriter
from java.lang import System
from java.io import FileInputStream, FileOutputStream, InputStreamReader
from javax.swing import JTextArea, JScrollPane

import pawt


# Execute an RDQL query.
def testQuery(queryString):
    query = Query(queryString)
    qe = QueryEngine(query)
    results = qe.exec()
    fmt = QueryResultsFormatter(results)

    sw = StringWriter()
    pw = PrintWriter(sw)
    fmt.printAll(pw, " | ")
    pw.flush()
    fmt.close()	
    results.close()

    txt = JTextArea(sw.toString(), 30, 60)
    pawt.test(JScrollPane(txt))

    
# Execute an RDQL on a given model.  The RDQL statement for this function
# function does not need the "FROM" clause (queries are assumed to be run
# against the given model).
def testOnModel(queryString, modelPath):
    model = ModelFactory.createDefaultModel()

    model.read(InputStreamReader(
               FileInputStream(modelPath)), "");
  
    query = Query(queryString)
    query.setSource(model)

    qe = QueryEngine(query)
    results = qe.exec()
    fmt = QueryResultsFormatter(results)

    sw = StringWriter()
    pw = PrintWriter(sw)
    fmt.printAll(pw, " | ")
    pw.flush()
    fmt.close()	
    results.close()

    txt = JTextArea(sw.toString(), 30, 60)
    pawt.test(JScrollPane(txt))


# Execute an RDQL query, but don't use the QueryResultsFormatter.
def testQueryWithoutQueryResultsFormatter(queryString):
    query = Query(queryString)
    qe = QueryEngine(query)
    results = qe.exec()

    outputStr = '' 
    for result in results:
        iter =  result.iterator()
        while iter.hasNext():
            outputStr += '%s: %s |  ' % (iter.varName(), iter.value())
            iter.next()
            
        outputStr += '\n'

    results.close() ;

    txt = JTextArea(outputStr, 30, 60)
    pawt.test(JScrollPane(txt))


if __name__ == '__main__':
    qString1 = \
    '''SELECT ?s, ?o
       WHERE  (?s, <http://www.w3.org/2001/vcard-rdf/3.0#NICKNAME>, ?o)'''
    testOnModel(qString1, 'mymodel.rdf')

    qString2 = \
    '''select ?s, ?p, ?o
       from <http://www.w3.org/1999/02/22-rdf-syntax-ns>
       where (?s, ?p, ?o)'''
    testQuery(qString2)

    qString3 = \
    '''select ?s, ?p, ?o
       from <http://www.w3.org/2000/01/rdf-schema>
       where (?s, ?p, ?o)'''
    testQueryWithoutQueryResultsFormatter(qString3)

            
